@@ -1,31 +1,33 @@
-/* main.c - MemTest-86  Version 3.0
+/* main.c - MemTest-86  Version 3.2
  *
  * Released under version 2 of the Gnu Public License.
- * By Chris Brady, cbrady@sgi.com
+ * By Chris Brady
+ * ----------------------------------------------------
+ * MemTest86+ V1.50 Specific code (GPL V2.0)
+ * By Samuel DEMEULEMEESTER, sdemeule@memtest.org
+ * http://www.x86-secret.com - http://www.memtest.org
  */
+
 #include "test.h"
 #include "defs.h"
 #undef TEST_TIMES
+#define DEFTESTS 11
 
 extern void bzero();
-extern int fast_mode;
 
 const struct tseq tseq[] = {
-	{0, 5, 3, 0, 0,    "[Address test, walking ones, no cache]"},
-	{1, 0, 3, 14, 0,   "[Moving inv, ones & zeros, cached]    "},
-	{0, 6, 3, 2, 0,    "[Address test, own address, no cache] "},
-	{1, 1, 2, 80, 0,   "[Moving inv, 8 bit pattern, cached]   "},
-	{1, 2, 2, 320, 0,  "[Moving inv, 32 bit pattern, cached]  "},
-	{1, 7, 64, 66, 0,  "[Block move, 64 moves, cached]        "},
-	{1, 3, 4, 240, 0,  "[Modulo 20, ones & zeros, cached]     "},
-	{0, 0, 2, 10, 0,   "[Moving inv, ones & zeros, no cache]  "},
-
-	{1, 7, 512, 514, 0,"[Block move, 512 moves, cached]       "},
-	{0, 1, 2, 80, 0,   "[Moving inv, 8 bit pattern, no cache] "},
-	{1, 4, 2, 1280, 0, "[Modulo 20, 8 bit pattern , cached]   "},
-	{0, 2, 2, 320, 0,  "[Moving inv, 32 bit pattern, no cache]"},
-	{1, 8, 1, 1, 0,    "[Bit fade test, 90 min, 2 patterns]   "},
-	{0, 0, 0, 0, 0, NULL}
+	{1,  5,  3,   0, 0, "[Address test, walking ones]          "},
+	{1,  6,  3,   2, 0, "[Address test, own address]           "},
+	{1,  0,  3,  14, 0, "[Moving inversions, ones & zeros]     "},
+	{1,  1,  2,  80, 0, "[Moving inversions, 8 bit pattern]    "},
+	{1, 10, 60, 300, 0, "[Moving inversions, random pattern]   "},
+	{1,  7, 64,  66, 0, "[Block move, 64 moves]                "},
+	{1,  2,  2, 320, 0, "[Moving inversions, 32 bit pattern]   "},
+	{1,  9, 40, 120, 0, "[Random number sequence]              "},
+	{1,  3,  4, 240, 0, "[Modulo 20, ones & zeros]             "},
+	{1,  8,  1,   2, 0, "[Bit fade test, 90 min, 2 patterns]   "},
+	{0,  4,  3,   2, 0, "[[Moving inversions, 0 & 1, uncached] "},
+	{0,  0,  0,   0, 0, NULL}
 };
 
 char firsttime = 0;
@@ -40,7 +42,7 @@ int test_ticks;
 int nticks;
 
 static int window = 0;
-static struct pmap windows[] = 
+static struct pmap windows[] =
 {
 	{ 0/* Written at startup */, 0x100000 },
 	{ 0, RES_START >> 12 },
@@ -120,7 +122,7 @@ static void run_at(unsigned long addr)
 
 	start = (unsigned long) &_start;
 	len = _end - _start;
-	if (	((start < addr) && ((start + len) >= addr)) || 
+	if (	((start < addr) && ((start + len) >= addr)) ||
 		((addr < start) &&  ((addr + len) >= start))) {
 		/* Handle overlap by doing an extra relocation */
 		if (addr + len < HIGH_TEST_ADR) {
@@ -135,7 +137,7 @@ static void run_at(unsigned long addr)
 
 void do_test(void)
 {
-	int i = 0, j = 0;
+	int i = 0;
 	unsigned long chunks;
 	unsigned long lo, hi;
 
@@ -151,7 +153,7 @@ void do_test(void)
 		if ((ulong)&_start != LOW_TEST_ADR) {
 			restart();
 		}
-		windows[0].start = 
+		windows[0].start =
 			( LOW_TEST_ADR + (_end - _start) + 4095) >> 12;
 		init();
 		firsttime = 1;
@@ -175,13 +177,13 @@ void do_test(void)
 	cprint(LINE_RANGE, COL_MID+14, " - ");
 	aprint(LINE_RANGE, COL_MID+17, hi);
 	aprint(LINE_RANGE, COL_MID+23, v->selected_pages);
-	cprint(LINE_RANGE, COL_MID+28, 
+	cprint(LINE_RANGE, COL_MID+28,
 		((ulong)&_start == LOW_TEST_ADR)?"          ":" Relocated");
-	
+
 #ifdef TEST_TIMES
 	{
 		ulong l, h, t;
-		
+
 		asm __volatile__ (
 			"rdtsc\n\t"
 			"subl %%ebx,%%eax\n\t"
@@ -208,32 +210,7 @@ void do_test(void)
 #endif
 	/* Now setup the test parameters based on the current test number */
 	/* Figure out the next test to run */
-	if (v->testsel < 0) {
-		switch(v->xtst_flag) {
-		case 0: /* Default tests */
-			if (v->test > DEFTESTS) {
-				goto skip_test;
-			}
-			break;
-		case 1: /* Extended tests */
-			if ((v->test <= DEFTESTS) || (v->test > DEFTESTS2)) {
-				goto skip_test;
-			}
-			break;
-		case 2: /* All tests */
-			if (v->test > DEFTESTS2) {
-				goto skip_test;
-			}
-			break;
-		}
-			
-		/* May skip this test if the cache settings have been */
-		/* overridden */
-		if ((v->cache_flag == 1 && tseq[v->test].cache == 0) ||
-			(v->cache_flag == 2 && tseq[v->test].cache == 1)) {
-			goto skip_test;
-		}
-	} else {
+	if (v->testsel >= 0) {
 		v->test = v->testsel;
 	}
 	dprint(LINE_TST, COL_MID+6, v->test, 2, 1);
@@ -247,7 +224,6 @@ void do_test(void)
 		len = v->map[i].end - v->map[i].start;
 		chunks += (len + SPINSZ -1)/SPINSZ;
 	}
-
 	test_ticks = find_ticks_for_test(chunks, v->test);
 	nticks = 0;
 	v->tptr = 0;
@@ -256,39 +232,32 @@ void do_test(void)
 
 	/* Now do the testing according to the selected pattern */
 	case 0:	/* Moving inversions, all ones and zeros */
-		p1 = 0;
-		p2 = ~p1;
-		movinv1(tseq[v->test].iter,p1,p2);
-		BAILOUT;
-	
-		/* Switch patterns */
-		p2 = p1;
-		p1 = ~p2;
-		movinv1(tseq[v->test].iter,p1,p2);
-		break;
-		
 	case 1: /* Moving inversions, 8 bit wide walking ones and zeros. */
-		p0 = 0x80;
-		for (i=0; i<8; i++, p0=p0>>1) {
-			p1 = p0 | (p0<<8) | (p0<<16) | (p0<<24);
-			p2 = ~p1;
-			movinv1(tseq[v->test].iter,p1,p2);
-			BAILOUT;
+	case 4:
 	
+		if (tseq[v->test].pat == 1)
+			p0 = 0x80808080;
+		else
+			p0 = 0;
+
+		for ( ; ; ) {
+			movinv1(tseq[v->test].iter,p0,~p0);
+			BAILOUT;
+
 			/* Switch patterns */
-			p2 = p1;
-			p1 = ~p2;
-			movinv1(tseq[v->test].iter,p1,p2);
+			movinv1(tseq[v->test].iter,~p0,p0);
 			BAILOUT
+			if ( !((unsigned char)(p0 >>= 1) & 0x7F) )
+				break;
 		}
 		break;
 
+
 	case 2: /* Moving inversions, 32 bit shifting pattern, very long */
 		for (i=0, p1=1; p1; p1=p1<<1, i++) {
 			movinv32(tseq[v->test].iter,p1, 1, 0x80000000, 0, i);
 			BAILOUT
-			movinv32(tseq[v->test].iter,~p1, 0xfffffffe,
-				0x7fffffff, 1, i);
+			movinv32(tseq[v->test].iter,~p1, 0xfffffffe, 0x7fffffff, 1, i);
 			BAILOUT
 		}
 		break;
@@ -308,38 +277,41 @@ void do_test(void)
 		}
 		break;
 
-	case 4: /* Modulo X check, 8 bit pattern */
-		p0 = 0x80;
-		for (j=0; j<8; j++, p0=p0>>1) {
-			p1 = p0 | (p0<<8) | (p0<<16) | (p0<<24);
-			for (i=0; i<MOD_SZ; i++) {
-				p2 = ~p1;
-				modtst(i, tseq[v->test].iter, p1, p2);
-				BAILOUT
-
-				/* Switch patterns */
-				p2 = p1;
-				p1 = ~p2;
-				modtst(i, tseq[v->test].iter, p1, p2);
-				BAILOUT
-			}
-			BAILOUT
-		}
-		break;
 	case 5: /* Address test, walking ones */
 		addr_tst1();
+		BAILOUT;
 		break;
 
 	case 6: /* Address test, own address */
 		addr_tst2();
+		BAILOUT;
 		break;
 
 	case 7: /* Block move test */
 		block_move(tseq[v->test].iter);
+		BAILOUT;
 		break;
-	
+
 	case 8: /* Bit fade test */
-		if (window == 0 ) { bit_fade(); }
+		if (window == 0 ) {
+			bit_fade();
+		}
+		BAILOUT;
+		break;
+		
+	case 9: /* Random Data Sequence */
+		for (i=0; i < tseq[v->test].iter; i++) {
+			movinvr();
+			BAILOUT;
+		}
+		break;
+	case 10: /* Random Data */
+		for (i=0; i < tseq[v->test].iter; i++) {
+			p1 = rand();
+			p2 = ~p1;
+			movinv1(2,p1,p2);
+			BAILOUT;
+		}
 		break;
 	}
  skip_window:
@@ -361,15 +333,15 @@ void do_test(void)
 		 *   The lower limit is less than START_ADR
 		 * - There is more than 1 meg of memory
 		 */
-		if (windows[window].start < 
+		if (windows[window].start <
 			(LOW_TEST_ADR + (_end - _start)) >> 12) {
-			if (v->pmap[v->msegs-1].end > 
+			if (v->pmap[v->msegs-1].end >
 				(((HIGH_TEST_ADR + (_end - _start)) >> 12) +1)) {
 				/* We need the high copy and we have enough
 				 * memory so use it.
 				 */
 				run_at(HIGH_TEST_ADR);
-			} else { 
+			} else {
 				/* We can't use this window so skip it */
 				goto skip_window;
 			}
@@ -385,7 +357,7 @@ void do_test(void)
 	skip_test:
 		v->test++;
 	bail_test:
-		/* Revert to the default mapping 
+		/* Revert to the default mapping
 		 * and enable the cache.
 		 */
 		paging_off();
@@ -394,7 +366,7 @@ void do_test(void)
 		window = 0;
 		cprint(LINE_PAT, COL_PAT-3, "   ");
 		/* If this was the last test then we finished a pass */
-		if (tseq[v->test].msg == NULL || v->testsel >= 0) {
+		if (v->test >= 9 || v->testsel >= 0) {
 			v->pass++;
 			dprint(LINE_INFO, COL_PASS, v->pass, 5, 0);
 			v->test = 0;
@@ -403,7 +375,7 @@ void do_test(void)
 			cprint(0, COL_MID+8,
 				"                                         ");
 		}
-		
+
 		/* We always start a pass with the low copy */
 		run_at(LOW_TEST_ADR);
 	}
@@ -414,17 +386,18 @@ void restart()
 	int i;
 	volatile char *pp;
 
-	/* clear all of the variables */
+	/* clear variables */
 	firsttime = 0;
-	fast_mode = 0;
-	for (i=0, pp=(char *)v; i<sizeof(struct vars); i++, pp++) {
-		*pp = 0;
-	}
+	v->test = 0;
+	v->pass = 0;
+	v->msg_line = 0;
+	v->ecount = 0;
+	v->ecc_ecount = 0;
 
-        /* Clear the screen */
-        for(i=0, pp=(char *)(SCREEN_ADR+0); i<80*24; i++, pp+=2) {
-                *pp = ' ';
-        }
+	/* Clear the screen */
+	for(i=0, pp=(char *)(SCREEN_ADR+0); i<80*24; i++, pp+=2) {
+		*pp = ' ';
+	}
 	run_at(LOW_TEST_ADR);
 }
 
@@ -448,25 +421,14 @@ void find_ticks(void)
 	}
 	compute_segments(window);
 	window = 0;
-	for (v->pass_ticks=0, i=0; tseq[i].msg != NULL; i++) {
+	for (v->pass_ticks=0, i=0; i<DEFTESTS != NULL; i++) {
 
 		/* Test to see if this test is selected for execution */
 		if (v->testsel >= 0) {
 			if (i != v->testsel) {
 				continue;
 			}
-		} else {
-			if (v->xtst_flag == 0 && i > DEFTESTS) {
-				break;
-			}
-			if (v->xtst_flag == 1 && i <= DEFTESTS) {
-				continue;
-			}
-			if ((v->cache_flag == 1 && tseq[i].cache == 0) ||
-				(v->cache_flag == 2 && tseq[i].cache == 1)) {
-				continue;
-			}
-                }
+		}
 		v->pass_ticks += find_ticks_for_test(chunks, i);
 	}
 }
@@ -546,11 +508,11 @@ static void compute_segments(int win)
 
 		cprint(LINE_SCROLL+(2*i+1), 44, "i=");
 		hprint(LINE_SCROLL+(2*i+1), 46, i);
-		
-		cprint(LINE_SCROLL+(2*i+2), 0, 
+
+		cprint(LINE_SCROLL+(2*i+2), 0,
 			"                                        "
 			"                                        ");
-		cprint(LINE_SCROLL+(2*i+3), 0, 
+		cprint(LINE_SCROLL+(2*i+3), 0,
 			"                                        "
 			"                                        ");
 #endif
@@ -566,5 +528,3 @@ static void compute_segments(int win)
 		}
 	}
 }
-
-
